# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('modeltranslation', '0003_auto_20151211_1232'),
    ]

    operations = [
        migrations.AlterIndexTogether(
            name='fieldtranslation',
            index_together=set([('source_md5',), ('model', 'object_id', 'lang', 'field', 'is_fuzzy')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FieldTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('model', models.CharField(max_length=128, verbose_name='Modelo a traducir')),
                ('object_id', models.PositiveIntegerField(default=1, help_text='Id del objeto traducido', verbose_name='Id del objeto')),
                ('field', models.CharField(help_text='Campo origen cuyo texto ha sido traducido', max_length=128, verbose_name='Campo origen')),
                ('lang', models.CharField(help_text='Idioma de la traducci\xf3n', max_length=16, verbose_name='Id del idioma', choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-cn', b'Simplified Chinese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'zh-tw', b'Traditional Chinese')])),
                ('source_text', models.TextField(verbose_name='Texto origen.')),
                ('source_md5', models.CharField(max_length=128, verbose_name='MD5 del texto origen.')),
                ('translation', models.TextField(default=None, help_text='Texto de la traducci\xf3n que se mostrar\xe1 al usuario cuando seleccione el idioma actual.', null=True, verbose_name='Traducci\xf3n')),
                ('is_fuzzy', models.BooleanField(default=False, help_text='Esta traducci\xf3n ha de ser revisada por un traductor.', verbose_name='\xbfRequiere revisi\xf3n?')),
                ('context', models.TextField(default=None, help_text='Contexto que ayudar\xe1 a los traductores a traducir este texto.', null=True, verbose_name='Contexto')),
                ('creation_datetime', models.DateTimeField(verbose_name='Fecha de creaci\xf3n del objeto')),
                ('last_update_datetime', models.DateTimeField(verbose_name='Fecha de \xfaltima actualizaci\xf3n del objeto')),
                ('creator_user', models.ForeignKey(related_name='model_translation', default=None, to=settings.AUTH_USER_MODEL, help_text='Usuario que ha realizado la \xfaltima traducci\xf3n', null=True, verbose_name='Usuario que ha realizado la traducci\xf3n')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-

###########################################
# Django models fields, etc.
from django.db import models

###########################################
# Users
from django.contrib.auth.models import User

###########################################
# Timezone methods
from django.utils import timezone

###########################################
# Global settings
from django.conf import settings

###########################################
# Used to MD5 computation
import hashlib

###########################################
# Used in module load and inspections
import importlib
import inspect
import sys

###########################################
# Middleware used to get current user
from cuser.middleware import CuserMiddleware

from django.contrib import admin

########################################################################
## Computes a checksum from an unicode or str.
def checksum(value):
	if type(value) == str:
		md5_value = hashlib.md5(value).hexdigest()
	else:
		md5_value = hashlib.md5(value.encode("utf-8")).hexdigest()
	return md5_value

########################################################################
## Languages
MODELTRANSLATION_LANG_CHOICES = settings.LANGUAGES

########################################################################################################################
## Gets the name of a translated attribute
def trans_attr(attr, lang):
	"""
	Returns the name of the translated attribute of the object <attribute>_<lang_iso_code>.
	For example: name_es (name attribute in Spanish)
	@param attr Attribute whose name will form the name translated attribute.
	@param lang ISO Language code that will be the suffix of the translated attribute.
	@return: string with the name of the translated attribute.
	"""
	lang = lang.replace("-","_").lower()
	return "{0}_{1}".format(attr,lang)


########################################################################################################################
## Gets the name of a fuzzy (incomplete) translated attribute
def trans_is_fuzzy_attr(attr,lang):
	"""
	Returns the name of the translated  is_fuzzy attribute: <attribute>_is_fuzzy_<lang_iso_code>.
	For example: name_is_fuzzy_es (is fuzzy name attribute in Spanish?)
	@param attr Attribute whose name will form the name translated attribute.
	@param lang ISO Language code that will be the suffix of the translated attribute.
	@return: string with the name of the translated attribute.
	"""
	return "{0}_is_fuzzy".format(trans_attr(attr,lang))


########################################################################
## Each one of the fields that is going to be translated in database
class FieldTranslation(models.Model):

	# Translatable modules (needs to be defined in settings.py)
	_model_module_paths = settings.TRANSLATABLE_MODEL_MODULES

	# Module cache, used to avoid importing several times the same module
	_modules = {}

	# Model cache that avoids importing more than once a Model
	_models = {}

	## Name of the Python module containing the model
	module = models.CharField(max_length=128, verbose_name=u"Module name", help_text=u"Module name that contains the model whose field is translated")

	## Name of the model whose field is translated
	model = models.CharField(max_length=128, verbose_name=u"Model name", help_text=u"Model name whose field is translated")

	## ID of the object whose field is translated
	object_id = models.PositiveIntegerField(default=1, verbose_name=u"Object id", help_text=u'Object id whose field is translated')

	## Name of the field that is going to be translated
	field = models.CharField(max_length=128, verbose_name=u"Object field", help_text=u'Name of the field that is translated')

	## Language used in the translation
	lang = models.CharField(max_length=16, choices=MODELTRANSLATION_LANG_CHOICES, verbose_name=u"Language Id", help_text=u'Language ISO code of this translation')

	## Source text
	source_text = models.TextField(verbose_name=u"Source text", help_text=u"Source text in default language")

	## MD5 of the source text (used to test changes)
	source_md5 = models.CharField(max_length=128, verbose_name=u"MD5 source text", help_text=u"MD5 checksum of source text")

	## Translation of the source_text
	translation = models.TextField(default=None, null=True, verbose_name=u"Translation", help_text=u"Translation showed to users in website when showing it in choosed language.")

	## Needs review? If True, it implies translation is not complete and will not be used.
	is_fuzzy = models.BooleanField(default=False, verbose_name=u"¿Needs reviewing?", help_text=u"This translation needs some reviewing.")

	## Context information that will help translators
	context = models.TextField(default=None, null=True, verbose_name=u"Context", help_text=u"Help context that will be helpful for translators.")

	## Creation date and time of the translation
	creation_datetime = models.DateTimeField(verbose_name=u"Creation date and time of this translation")

	## Last update date and time of the translation
	last_update_datetime = models.DateTimeField(verbose_name=u"Last update date and time of this translation")

	## Creator user
	creator_user = models.ForeignKey(User, null=True, default=None, related_name='model_translation', verbose_name=u"User translator", help_text=u"User that created last translation version")

	## Metainformation of FieldTranslation
	class Meta:
		verbose_name = u"model object field translation"
		verbose_name_plural = u"model object field translations"
		index_together = [
		    ["module", "model", "object_id", "lang", "field", "is_fuzzy"],
		    ["lang", "is_fuzzy"],
			["source_md5"]
		]


	####################################################################################################################
	## Conversion of an object FieldTranslation to unicode
	def __unicode__(self):
		is_fuzzy_text = "*" if self.is_fuzzy else ""
		return u"{0} {1}-{2}-{3}-{4}-'{5}'-'{6}'{7}".format(self.id,self.model,self.object_id,self.field,self.lang,self.source_text,self.translation,is_fuzzy_text)


	####################################################################################################################
	## Conversion of an object FieldTranslation to str
	def __str__(self):
		is_fuzzy_text = "*" if self.is_fuzzy else ""
		return "{0}-{1}-{2}-{3}-'{4}'-'{5}'{6}".format(self.model,self.object_id,self.field,self.lang,self.source_text,self.translation,is_fuzzy_text)


	####################################################################################################################
	## Caching of translatable modules
	@staticmethod
	def _init_module_cache():
		"""
		Module caching, it helps with not having to import again and again same modules.
		@return: boolean, True if module  caching has been done, False if module caching was already done.
		"""

		# While there are not loaded modules, load these ones
		if len(FieldTranslation._modules) < len(FieldTranslation._model_module_paths):
			for module_path in FieldTranslation._model_module_paths:
				FieldTranslation._modules[module_path] = importlib.import_module(module_path)
			return True
		return False


	####################################################################################################################
	## Gets the Python module associated to this field translation
	def get_python_module(self):
		if not self.module in FieldTranslation._modules:
			FieldTranslation._modules[self.module] = importlib.import_module(self.module)
		return FieldTranslation._modules[self.module]


	####################################################################################################################
	## Loads source model in a dynamic attribute in this object
	def _load_source_model(self):
		"""
		Loads and gets the source model of the FieldTranslation as a dynamic attribute. It is used only when deleting
		orphan translations (translations without a parent object associated).
		"""

		# If source_model exists, return it
		if hasattr(self, "source_model"):
			return self.source_model

		# Getting the source model
		module = self.get_python_module()

		# Test if module has inside the model we are looking for
		if hasattr(module, self.model):
			# Setting of source model
			self.source_model = getattr(module, self.model)
			# Setting of verbose name and its plural for later use
			self.source_model.meta__verbose_name = self.source_model._meta.verbose_name
			self.source_model.meta__verbose_name_plural = self.source_model._meta.verbose_name_plural
			return self.source_model
		
		raise ValueError(u"Model {0} does not exist in module".format(self.model, self.module))


	####################################################################################################################
	## Loads source object in a dynamic attribute in this object
	def _load_source_object(self):
		"""
		Loads related object in a dynamic attribute and returns it.
		"""
		if hasattr(self, "source_obj"):
			self.source_text = getattr(self.source_obj, self.field)
			return self.source_obj

		self._load_source_model()
		self.source_obj = self.source_model.objects.get(id=self.object_id)
		return self.source_obj


	####################################################################################################################
	## Delete orphan translations (translations that have no parent object)
	@staticmethod
	def delete_orphan_translations(condition=None):
		"""
		Delete orphan translations.
		This method needs refactoring to be improve its performance.
		"""
		if condition is None:
			condition = {}
		# TODO: optimize using one SQL sentence
		translations = FieldTranslation.objects.all()
		for translation in translations:
			translation._load_source_model()
			condition["id"] = translation.object_id
			if not translation.source_model.objects.filter(**condition).exists():
				translation.delete()


	####################################################################################################################
	## Creates new entries in FieldTranslations table based on new objects not yet inserted
	@staticmethod
	def update_translations(condition=None):
		"""
		Updates FieldTranslations table
		"""
		if condition is None:
			condition = {}

		# Number of updated translations
		num_translations = 0
		
		# Module caching
		FieldTranslation._init_module_cache()

		# Current languages dict
		LANGUAGES = dict(lang for lang in MODELTRANSLATION_LANG_CHOICES)
		if settings.LANGUAGE_CODE in LANGUAGES:
			del LANGUAGES[settings.LANGUAGE_CODE]

		# For each module, we are going to update the translations
		for key in FieldTranslation._modules.keys():
			module = FieldTranslation._modules[key]

			# Class of the module
			clsmembers = inspect.getmembers(sys.modules[key], inspect.isclass)
			for cls in clsmembers:
				cls = cls[1]

				# If the model has in Meta "translatable_fields", we insert this fields
				if hasattr(cls,"_meta") and not cls._meta.abstract and hasattr(cls._meta,"translatable_fields") and len(cls._meta.translatable_fields)>0:
					objects = cls.objects.filter(**condition)
					
					# For each object, language and field are updated
					for obj in objects:
						for lang in LANGUAGES.keys():
							for field in cls._meta.translatable_fields:
								if FieldTranslation.update(obj=obj, field=field, lang=lang, context=""):
									num_translations += 1
		return num_translations


	####################################################################################################################
	## Returns source object
	def get_source_obj(self):
		"""
		Returns source object.
		"""
		return self._load_source_object()


	####################################################################################################################
	## Returns source model
	def get_source_model(self):
		"""
		Returns source model
		"""
		return self._load_source_model()


	####################################################################################################################
	## Returns source model field name
	def get_verbose_field_name(self):
		self._load_source_model()
		try:
			verbose_field_name = self.source_model._meta.get_field_by_name(self.field)[0].verbose_name
			return verbose_field_name
		except:
			return u"Error loading field name"


	####################################################################################################################
	## Constructs a new FieldTranslation object
	@staticmethod
	def factory(obj, field, source_text, lang, context=""):
		"""
		Static method that constructs a translation based on its contents.
		"""

		# Model name
		obj_classname = obj.__class__.__name__

		# Module name
		obj_module = obj.__module__
		
		# Computation of MD5 of the source text
		source_md5 = checksum(source_text)

		# Translated text
		translation = ""

		# Translation text attribute name
		field_lang = trans_attr(field,lang)

		# If in source object there is a translation and is not empty, we asume it was assigned and it's correct
		if hasattr(obj,field_lang) and getattr(obj,field_lang)!="":
			translation = getattr(obj,field_lang)

		# Is the translation fuzzy?
		is_fuzzy = True
		is_fuzzy_lang = trans_is_fuzzy_attr(field,lang)
		if hasattr(obj,is_fuzzy_lang):
			is_fuzzy = getattr(obj,is_fuzzy_lang)

		# Construction of the translation
		trans = FieldTranslation(module=obj_module, model=obj_classname, object_id=obj.id,
		                         field=field, lang=lang, source_text=source_text, source_md5=source_md5,
		                         translation=translation, is_fuzzy=is_fuzzy, context=context)
		return trans


	####################################################################################################################
	## Updates this translation
	def _update(self, obj, field, source_text, context=""):
		#### Update 1: "is_fuzzy" update
		# is_fuzzy field
		lang = self.lang
		is_fuzzy = False
		is_fuzzy_lang = trans_is_fuzzy_attr(field,lang)
		# If source object has is_fuzzy_lang attribute for this field, we get it
		if hasattr(obj, is_fuzzy_lang):
			is_fuzzy = getattr(obj, is_fuzzy_lang)
			self.is_fuzzy = is_fuzzy

		#### Update 2: Update object
		# Source text and source MD5
		self.source_text = source_text
		self.source_md5 = checksum(source_text)

		# Translated text
		# Maybe source object has the translated text. In that case, we get it
		field_lang = trans_attr(field,lang)
		if hasattr(obj,field_lang):
			translation = getattr(obj, field_lang)
			# Is the translation different? In that case, changes the context
			if self.translation != translation:
				self.context = context
			# Translation is alway updated
			self.translation = translation

		# If translation is empty, is_fuzzy must be True always.
		# We don't want to show empty translations in our website
		if self.translation == "":
			self.is_fuzzy = True
		
		self.save()
		return self


	####################################################################################################################
	## Updates a translation
	@staticmethod
	def update(obj, field, lang, context=""):
		try:
			# Class name
			obj_classname = obj.__class__.__name__
			# Module name
			obj_module = obj.__module__
			# Updating of the translation
			trans = FieldTranslation.objects.get(module=obj_module, model=obj_classname, object_id=obj.id, field=field, lang=lang)
			return trans._update(obj=obj, field=field, source_text=getattr(obj,field), context=context)

		# In case the translation does not exist, create a new one
		except FieldTranslation.DoesNotExist:
			source_text = getattr(obj,field)
			# Updates translation only when attribute is a string
			if not source_text is None:
				trans = FieldTranslation.factory(obj=obj, field=field, source_text=getattr(obj,field), lang=lang, context=context)
				return trans._update(obj=obj, field=field, source_text=getattr(obj,field), context=context)
			return False


	####################################################################################################################
	## Save object in database, updating the datetimes accordingly
	def save(self, *args, **kwargs):
		"""
		Save object in database, updating the datetimes accordingly.
		
		"""
		# Now in UTC
		now_datetime = timezone.now()

		# If we are in a creation, assigns creation_datetime
		if not self.id:
			self.creation_datetime = now_datetime

		# Las update datetime is always updated
		self.last_update_datetime = now_datetime
		
		# Current user is creator
		# (get current user with django-cuser middleware)
		self.creator_user = None
		current_user = CuserMiddleware.get_user()
		if not current_user is None and not current_user.is_anonymous():
			self.creator_user_id = current_user.id
		
		# Parent constructor call
		super(FieldTranslation, self).save(*args, **kwargs)


# Admin registration of FieldTranslation model
admin.site.register(FieldTranslation)


# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-01-19 13:25
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('modeltranslation', '0007_auto_20160119_1351'),
    ]

    operations = [
        migrations.AlterIndexTogether(
            name='fieldtranslation',
            index_together=set([('source_md5',), ('lang', 'is_fuzzy'), ('module', 'model', 'object_id', 'lang', 'field', 'is_fuzzy')]),
        ),
    ]



# -*- coding: utf-8 -*-

########################################################################################################################
########################################################################################################################
#### THIS IS NOT USED. FUTURE VERSIONS WILL IMPROVE PERFORMANCE BY USE OF THIS CACHE OF TRANSLATIONS
########################################################################################################################
########################################################################################################################


###########################################
# Lo básico sobre los modelos
from django.db import models
from django.utils import timezone

## Clase singleton que tiene 
class TransCache:
    
    ## Diccionario que contiene la caché
    cache = {}
    
    ## Fecha de creación de la caché
    SINGLETON_CREATION_DATETIME = None
    
    ## Tiempo de vida de la caché
    SINGLETON_EXPIRATION_MAX_SECONDS = 7200
    
    ## Tamaño de la caché
    SINGLETON_SIZE = 1000
    
    ## Tamaño de la caché
    SINGLETON = None
    
    @staticmethod
    def _create_key(lang, instance):
        """Crea la clave única de la caché"""
        model_name = instance.__class__.__name__
        return "{0}__{1}_{2}".format(lang,model_name,instance.id)

    @staticmethod
    def _cache_is_expired():
        """Indica si la caché está caducada"""
        now = timezone.now()
        timediff = TransCache.SINGLETON_CREATION_DATETIME - now
        return (timediff.total_seconds() > TransCache.SINGLETON_EXPIRATION_MAX_SECONDS)
    
    def _cache_is_too_big(self):
        """Indica si la caché es demasiado grande"""
        return len(self.cache) > TransCache.SINGLETON_SIZE
    
    def __init__(self, *args, **kwargs):
        """Inicializa la caché de traducciones"""
        self.cache = {}
    
    @staticmethod
    def factory():
        """Factoría del singleton, o crea una nueva o devuelve la existente"""
        if not TransCache.SINGLETON or TransCache._cache_is_expired():
            TransCache.SINGLETON = TransCache()
            TransCache.SINGLETON_CREATION_DATETIME = timezone.now()

        return TransCache.SINGLETON

    def clear(self):
        """Limpia la caché"""
        self.cache = {}
        self.creation_datetime = timezone.now()

    def set(self, lang, instance):
        """
        Establece en la instancia actual los atributos de traducción
        y la almacena en un diccionario de claves _create_key y valores
        el objeto con los atributos dinámicos.
        """
        if self._cache_is_too_big():
            self.cache = {}
        instance_key = TransCache._create_key(lang, instance)
        instance._translations_are_cached = True
        instance.load_translations(lang=lang)
        self.cache[instance_key] = instance
        
    def has(self, lang, instance):
        """
        Indica si la caché tiene un objeto igual a éste con los
        atributos dinámicos de traducción
        """
        instance_key = TransCache._create_key(lang, instance)
        return instance_key in self.cache

    def get(self, lang, instance):
        """
        Obtiene una instancia igual a ésta, pero con los atributos
        dinámicos de traduccción
        """
        instance_key = TransCache._create_key(lang, instance)
        return self.cache[instance_key]




# -*- coding: utf-8 -*-

from django import template
from django.conf import settings
from django.utils.translation import get_language

from modeltranslation.translation import _get_fieldtranslations

register = template.Library()

########################################################################
## Gets current language translations
## allowing developers to call {{ <obj>|_:"<attr>" }} that translates
## attribute <attr> to current language. For example: {{ event|_:"name" }}
def get_translated_attribute(instance, attr):
	"""
	Wraps Django Model __getattribute__ method making translation in templates less painful
	"""
	
	# If its class has no translatable fields, returns attribute
	try:
		if not hasattr(instance._meta, "translatable_fields") or len(getattr(instance._meta,"translatable_fields"))==0:
			return getattr(instance, attr)
	except AttributeError:
		return instance
	
	# Translatable fields of this instance
	translatable_fields = instance._meta.translatable_fields

	# Current language
	cur_language = get_language()
	lang = cur_language.title().lower()

	# If current language is default language, returns attribute
	if lang == settings.LANGUAGE_CODE:
		return getattr(instance, attr)
	
	# Otherwise, if a translation is NOT needed for attr atribute, get attribute
	if not attr in translatable_fields:
		return getattr(instance, attr)

	# Gets field translations of this instance and return the translated attribute
	field_translation = _get_fieldtranslations(instance, field=attr, lang=lang)
	if field_translation:
		if not field_translation.is_fuzzy:
			return field_translation.translation
	return getattr(instance, attr)


# Register this template filter
register.filter('_', get_translated_attribute)



# -*- coding: utf-8 -*-

from modeltranslation.models import checksum, FieldTranslation, trans_attr, trans_is_fuzzy_attr
from django.db import models
from django.db.models import signals
from django.conf import settings
from django.utils import translation
import sys
import inspect


########################################################################
########################################################################
#	ATENCIÓN, LÉEME:
#	Instrucciones:
#	1.	Añadir "from modeltranslation.translation import addtranslations"
#		en los modelos de tu aplicación.
#	2.	Añade a cada modelo de tu aplicación un campo translatable_fields
#		en el Meta con una lista de los atributos a traducir.
#	3.	Realiza esta llamada al FINAL del archivo de modelos:
#		addtranslations(__name__)
########################################################################
########################################################################

########################################################################
########################################################################
#	WARNING, README FIRST:
#	Instructions:
#	1.	Add "from modeltranslation.translation import addtranslations"
#		in your application models.
#	2.	Add to each model you want to translate a new field in Meta with
#		the name "translatable_fields". This field must contain a list
#       of fields that needs to be translated.
#	3.	Call this function AT THE END of your models.py:
#		addtranslations(__name__)
########################################################################
########################################################################


########################################################################
## Add a new field to Meta options "translatable_fields"
models.options.DEFAULT_NAMES += ('translatable_fields',)


########################################################################
## Returns True if there is only one language in this website
def site_is_monolingual():
	return (hasattr(settings, "IS_MONOLINGUAL") and settings.IS_MONOLINGUAL) or len(settings.LANGUAGES)==0


########################################################################
## Adds a signal that will be triggered when saving a model object
def _save_translations(sender, instance, *args, **kwargs):
	"""
	This signal saves model translations.
	"""
	
	# If we are in a site with one language there is no need of saving translations
	if site_is_monolingual():
		return False

	cls = sender

	# If its class has no "translatable_fields" then there are no translations
	if not hasattr(cls._meta, "translatable_fields"):
		return False

	# For each translatable field, get its value, computes its md5 and for each language creates its
	# empty translation.
	for field in cls._meta.translatable_fields:
		value = getattr(instance,field)
		if not value is None:
			md5_value = checksum(value)
			setattr( instance, u"md5"+field, md5_value )
			for lang in settings.LANGUAGES:
				lang = lang[0]
				# print "({0}!={1}) = {2}".format(lang, settings.LANGUAGE_CODE,lang!=settings.LANGUAGE_CODE)
				if lang != settings.LANGUAGE_CODE:
					context = u"Updating from object"
					if hasattr(instance, "trans_context"):
						context = getattr(instance, "trans_context")
					trans = FieldTranslation.update(instance, field, lang, context)


########################################################################
########################################################################
########################################################################
## GET TRANSLATIONS ##


########################################################################################################################
## Get translated fields
def _get_fieldtranslations(instance, field=None, lang=None):
	"""
	Get all the translations for this object.
	"""

	# Basic filtering: filter translations by module, model an object_id
	_filter = {"module": instance.__module__, "model": instance.__class__.__name__, "object_id": instance.id}

	if lang:
		_filter["lang"] = lang

	# If we specify a field we get ONE translation (this field translation)
	if field:
		_filter["field"] = field
		try:
			return FieldTranslation.objects.get(**_filter)
		except FieldTranslation.DoesNotExist:
			return False

	# Otherwise, we get all translations
	return FieldTranslation.objects.filter(**_filter)


########################################################################################################################
## Load translations of an instance
def _load_translations(instance, lang=None):
	"""
	Loads all translations as dynamic attributes:
		<attr>_<lang_code>
		<attr>_<lang_code>_is_fuzzy
	"""
	# Gets field translations
	translations = _get_fieldtranslations(instance=instance, lang=lang)
	for translation_i in translations:
		# Sets translated field lang for this language
		field_lang = trans_attr(translation_i.field, translation_i.lang)
		setattr(instance, field_lang, translation_i.translation)
		# Sets is_fuzzy value for this language
		field_isfuzzy_lang = trans_is_fuzzy_attr(translation_i.field, translation_i.lang)
		#print "{0} ({1}), attr_name={2} value={3}".format(translation.field, translation.lang, isfuzzy_lang,translation.is_fuzzy)
		setattr(instance, field_isfuzzy_lang, translation_i.is_fuzzy)
	return True


########################################################################################################################
## Sets translations from a dict. Used in ModelForms
def _set_dict_translations(instance, dict_translations):
	"""
	Establece los atributos de traducciones a partir de una dict que
	contiene todas las traducciones.
	"""

	# If class has no translatable fields get out
	if not hasattr(instance._meta, "translatable_fields"):
		return False

	# If we are in a site with one language there is no need of saving translations
	if site_is_monolingual():
		return False

	# Translatable fields
	translatable_fields = instance._meta.translatable_fields

	# For each translatable field and for each language (excluding default language), we have to see if there is
	# two dynamic fields:
	# - <attribute>_<language_code>: translated atribute in <language_code>.
	#   For example: name_fr, name_es, description_it, etc.
	# - <attribute>_is_fuzzy_<language_code>: is a provisional translation of <attribute> for language <language_code>.
	#   For example: name_is_fuzzy_fr, name_is_fuzzy_es, description_is_fuzzy_it, etc.
	for field in translatable_fields:
		for lang in settings.LANGUAGES:
			lang = lang[0]
			if lang != settings.LANGUAGE_CODE:
					
				# Translated field name
				trans_field = trans_attr(field,lang)
				# If translated field name is in the dict, we assign it to the object
				if dict_translations.has_key(trans_field):
					setattr(instance,trans_field,dict_translations[trans_field])
					
				# Is fuzzy attribute
				trans_isfuzzy = trans_is_fuzzy_attr(field,lang)
				# If "is fuzzy" name is in the dict, we assign it to the object
				if dict_translations.has_key(trans_isfuzzy):
					is_fuzzy_value = (dict_translations[trans_isfuzzy]=="1") or (dict_translations[trans_isfuzzy]==1)
					setattr(instance,trans_isfuzzy, is_fuzzy_value)


########################################################################
## Gets the translated field of an instance
def _get_translated_field(instance, attr):
	# If we don't have translations, return the original attribute
	if site_is_monolingual():
		return getattr(instance, attr)
	
	# Current language
	lang = translation.get_language()
	#print u"\n{0}[id={1}]: {2}='{3}', atrr_in_lang='{2}_{4}'".format(instance.__class__.__name__, instance.id, attr, getattr(instance,attr), lang)
	
	# Load translations
	_load_translations(instance=instance, lang=lang)
	
	# Names of translated attributes of translated attribute and related is_fuzzy one.
	trans_field = trans_attr(attr, lang)
	is_fuzzy_attr = trans_is_fuzzy_attr(attr, lang)
	
	# If the translated attribute exists and is not fuzzy, return it
	if hasattr(instance,trans_field) and hasattr(instance, is_fuzzy_attr) and not getattr(instance, is_fuzzy_attr):
		return getattr(instance,trans_field)
	
	# Otherwise, return its original value
	return getattr(instance, attr)


########################################################################
########################################################################
########################################################################
## HANDLERS ##


########################################################################
## Adds the actions to a class
def add_translation(sender):
	"""
	Adds the actions to a class.
	"""
	# 1. Execute _save_translations when saving an object
	signals.post_save.connect(_save_translations, sender=sender)
	# 2. Adds get_fieldtranslations to class. Remember that this method get all the translations.
	sender.add_to_class("get_fieldtranslations", _get_fieldtranslations)
	# 3. Adss load_translations. Remember that this method included all the translations as dynamic attributes.
	sender.add_to_class("load_translations", _load_translations)
	# 4. Adds _set_dict_translations. This methods allows us setting all the translated fields form a dict.
	# Very useful when dealing with ModelForms.
	sender.add_to_class("set_translation_fields", _set_dict_translations)
	# 5. This methods returns one translated attribute in Django.
	# Avoid using _ and use get_trans_attr because Django maketranslations parser is fooled believing that everything
	# inside _ methods is translatable.
	sender.add_to_class("_", _get_translated_field)
	sender.add_to_class("get_trans_attr", _get_translated_field)
	sender.add_to_class("_t", _get_translated_field)


########################################################################
## Adds translation actions to a module
def addtranslations(module=__name__):
	# For each class in that module
	clsmembers = inspect.getmembers(sys.modules[module], inspect.isclass)
	for cls in clsmembers:
		cls = cls[1]
		# If this class has translatable fields, add actions to it
		if hasattr(cls,"_meta") and not cls._meta.abstract and hasattr(cls._meta,"translatable_fields") and len(cls._meta.translatable_fields)>0:
			add_translation(cls)





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('modeltranslation', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fieldtranslation',
            name='lang',
            field=models.CharField(help_text='Idioma de la traducci\xf3n', max_length=16, verbose_name='Id del idioma', choices=[('es', 'Espa\xf1ol'), ('en', 'English'), ('de', 'Deutsch')]),
        ),
    ]



# -*- coding: utf-8 -*-

import hashlib

from django.shortcuts import render_to_response, render, get_object_or_404
from django.template import RequestContext
from django.http import HttpResponseRedirect, Http404
from django.core.urlresolvers import reverse
from django.db import transaction
from django.conf import settings
import re

from modeltranslation.admin.forms import FieldTranslationForm, ImportTranslationsForm
from modeltranslation.models import FieldTranslation


########################################################################
########################################################################
# Admin index
def admin(request):
	return render_to_response('modeltranslation/admin/admin.html',{}, RequestContext(request))


########################################################################
########################################################################
# Ver todas las traducciones
def view_all(request, language, filter=None):
	"""
	View all translations that are in site.
	"""

	# Is there any filter?
	if request.method == "POST":
		data = request.POST.dict()

		if not data["search"] or data["search"]=="":
			return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(data["language"],data["filter"])))

		return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(data["language"],data["filter"]))+"?search="+data["search"])

	LANGUAGES = dict(lang for lang in settings.LANGUAGES)
	if language not in LANGUAGES.keys():
		raise Http404(u"Language {0} does not exist".format(language))
	
	if language == settings.LANGUAGE_CODE:
		raise Http404(u"El idioma {0} es el idioma por defecto".format(language))

	# Translation filter
	trans_filter = {"lang":language}
	if filter == "all" or filter is None:
		pass
	elif filter == "fuzzy":
		trans_filter["is_fuzzy"] = True
	elif filter == "completed":
		trans_filter["is_fuzzy"] = False
	
	search_query = ""
	if request.GET and "search" in request.GET and request.GET.get("search")!="":
		search_query = request.GET.get("search")
		trans_filter["source_text__icontains"] = search_query

	translations = FieldTranslation.objects.filter(**trans_filter)
	
	# Update translations
	active_translations = []
	for translation in translations:
		source_model = translation.get_source_model()
		if not translation.field in source_model._meta.translatable_fields:
			translation.delete()
		else:
			active_translations.append(translation)
	
	replacements = {"translations":active_translations, "filter":filter, "lang":language, "language":LANGUAGES[language], "search_query":search_query}
	return render_to_response('modeltranslation/admin/list.html',replacements, RequestContext(request))


########################################################################
########################################################################
# Edit a translation
def edit(request, translation):
	"""
	Edit a translation.
	@param request: Django HttpRequest object.
	@param translation: Translation id
	@return Django HttpResponse object with the view or a redirection.
	"""
	translation = get_object_or_404(FieldTranslation, id=translation)
	if request.method == 'POST':
		if "cancel" in request.POST:
			return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(translation.lang,"all")))
		elif "save" in request.POST:
			form = FieldTranslationForm(request.POST, instance=translation)
			valid_form = form.is_valid()
			if valid_form:
				translation = form.save(commit=False)
				translation.context = u"Admin. Traducciones"
				translation.save()
				return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(translation.lang,"all")))
		else:
			form = FieldTranslationForm(instance=translation)
	else:
		form = FieldTranslationForm(instance=translation)

	LANGUAGES = dict(lang for lang in settings.LANGUAGES)
	language = LANGUAGES[translation.lang]
	
	return render_to_response('modeltranslation/admin/edit_translation.html',{"translation":translation, "form":form, "lang":translation.lang, "language":language}, RequestContext(request))


########################################################################
########################################################################
## Import translation in .po format
@transaction.atomic
def import_translations(request, language):
	"""
	Import translations froma PO file. Please take note that this PO file MUST be generated with this application
	because translation comments in the file are used as translation ids.
	"""
	def _import_po_file(uploadedfile, lang):
		lines = []
		for line in uploadedfile:
			lines.append(line)
		num_lines = len(lines)
		
		prog_ctxt = re.compile(r"msgctxt\s+\"(?P<id>\d+)--(?P<module>(\.\w)+)--(?P<model>\w+)--(?P<object_id>\d+)--(?P<field>\w+)\"")
		prog_msgid = re.compile(r"msgid\s+\"(?P<source_text>.+)\"$")
		prog_msgstr = re.compile(r"msgstr\s+(?P<trans>.+)")
		
		i = 0
		while i < num_lines:
			line = lines[i]
			result = prog_ctxt.match(line)
			if result:
				id = result.group("id")
				is_fuzzy = (lines[i-1] == "#, fuzzy\n")

				source_text = lines[i+1]
				translation_line = lines[i+2]

				# Traducción
				g = prog_msgstr.match(translation_line)
				if g is None:
					i += 1
					continue
				translation = g.group("trans").replace("msgstr","")[1:-1].replace("\\\"","\"").replace('\\\'','\'')
					
				# Get translation from a translation id
				try:
					field_trans = FieldTranslation.objects.get(id=id)
				except FieldTranslation.DoesNotExist:
					source_text = source_text.replace("msgid","")[1:-1].replace("\\\"","\"").replace('\\\'','\'')
					source_md5 = hashlib.md5(source_text.encode("utf-8")).hexdigest()
					field_trans = FieldTranslation(
						module=result.group("module"), model=result.group("model"), object_id=result.group("object_id"),
						field=result.group("field"), lang=lang, source_text=source_text, source_md5=source_md5
					)
					
				# Sets translation and is_fuzzy attribute
				field_trans.translation = translation
				field_trans.is_fuzzy = is_fuzzy
				field_trans.save()
				i += 4
			i += 1
	
	# Delete orphan translations
	FieldTranslation.delete_orphan_translations()
	
	if request.method != "POST":
		return HttpResponseRedirect(reverse("modeltranslation:admin_url"))
	
	form = ImportTranslationsForm(request.POST, request.FILES)
	if form.is_valid():
		_import_po_file(request.FILES['file'], language)
		return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(language,"all")))

	return HttpResponseRedirect(reverse("modeltranslation:admin_url"))


########################################################################
########################################################################
## Export translation in PO format
def export_translations(request, language):
	"""
	Export translations view.
	"""
	FieldTranslation.delete_orphan_translations()
	translations = FieldTranslation.objects.filter(lang=language)
	for trans in translations:
		trans.source_text = trans.source_text.replace("'","\'").replace("\"","\\\"")
		trans.translation = trans.translation.replace("'","\'").replace("\"","\\\"")
	replacements = {"translations":translations, "lang":language}
	if len(settings.ADMINS)>0:
		replacements["last_translator"] = settings.ADMINS[0][0]
		replacements["last_translator_email"] = settings.ADMINS[0][1]
	if settings.WEBSITE_NAME:
		replacements["website_name"] = settings.WEBSITE_NAME
	response = render(request=request, template_name='modeltranslation/admin/export_translations.po', dictionary=replacements, context_instance=RequestContext(request), content_type="text/x-gettext-translation")
	response['Content-Disposition'] = 'attachment; filename="{0}.po"'.format(language)
	return response


########################################################################
########################################################################
## Update translations
def update_translations(request):
	"""
	Update translations: delete orphan translations and creates empty translations for new objects in database.
	"""
	FieldTranslation.delete_orphan_translations()
	num_translations = FieldTranslation.update_translations()
	return render_to_response('modeltranslation/admin/update_translations_ok.html',{"num_translations":num_translations}, RequestContext(request))
	



# -*- coding: utf-8 -*-

########################################################################
##
## Formularios para la traducción masiva de textos
##
########################################################################

from django import forms

from modeltranslation.models import FieldTranslation

from tinymce.widgets import TinyMCE

import re

def has_html_tag(text):
	result = re.match(r"<p>|<ul>|<img>|<strong>|<span>", text)
	return result


class ModelFormTrimForm(forms.ModelForm):

	def clean(self):
		cleaned_data = super(ModelFormTrimForm, self).clean()

		for field in self.cleaned_data:
			if isinstance(self.cleaned_data[field], basestring):
				self.cleaned_data[field] = self.cleaned_data[field].strip()

		#Hay que devolver siempre el array "cleaned_data"
		return cleaned_data


class FieldTranslationForm(ModelFormTrimForm):
	class Meta:
		model = FieldTranslation
		exclude = ("module", "model", "object_id", "field", "lang", "source_text", "source_md5", "context", "creation_datetime", "last_update_datetime", "creator_user")

	class Media:
		css = {
			"all": ("css/modeltranslation/forms/common.css",)
		}

		js = (
			"js/modeltranslation/forms/common.js",
		)

	def __init__(self, *args, **kwargs):
		super(FieldTranslationForm, self).__init__(*args, **kwargs)
		if self.instance and has_html_tag(self.instance.source_text):
			self.fields["translation"].widget = TinyMCE()

	def clean(self):
		cleaned_data = super(FieldTranslationForm, self).clean()
		return cleaned_data
		
	def save(self, *args, **kwargs):
		obj = super(FieldTranslationForm, self).save(*args, **kwargs)
		return obj


########################################################################
########################################################################
## Actualizador del archivo .po
class ImportTranslationsForm(forms.Form):
	file = forms.FileField()



# -*- coding: utf-8 -*-

from modeltranslation.models import FieldTranslation
import hashlib

###########################################
## Configuraciones del sistema
from django.conf import settings

def _make_translations(obj):
	cls = obj.__class__
	obj.classname = cls.__name__
	if hasattr(cls,"TRANSLATABLE_FIELDS"):
		for field in cls.TRANSLATABLE_FIELDS:
			value = getattr(obj,field)
			md5_value = hashlib.md5(value.encode("utf-8")).hexdigest()
			setattr( obj, "md5"+field, md5_value )
			for lang in settings.LANGUAGES:
				trans = FieldTranslation.update(obj, field, lang[0])


def savetranslation(fn):
	def _make_translations_wrapper(self):
		_make_translations(self)
		fn(self)
	return _make_translations_wrapper

#def trace(cls):
	#for name, m in inspect.getmembers(cls, inspect.ismethod):
		#print cls
		#print "- "cls
		#setattr(cls,name,log(m))
	#return cls



# -*- coding: utf-8 -*-

from django.shortcuts import render_to_response, render, get_object_or_404
from django.template import RequestContext
from django.http import HttpResponseRedirect, Http404

from django.core.urlresolvers import reverse
from django.db import transaction
from modeltranslation.forms import FieldTranslationForm, ImportTranslationsForm

from modeltranslation.models import checksum, FieldTranslation, trans_attr, trans_is_fuzzy_attr

from django.conf import settings

import re
import hashlib

########################################################################
########################################################################
# Index de administración 
def admin(request):
	return render_to_response('modeltranslation/admin/admin.html',{}, RequestContext(request))


########################################################################
########################################################################
# Ver todas las traducciones
def view_all(request, language, filter=None):
	"""
	Visualización del listado de traducciones.
	Muestra todas las traducciones existentes en el sistema.
	"""

	# Si se ha enviado una búsqueda por POST,
	if request.method == "POST":
		data = request.POST.dict()

		if not data["search"] or data["search"]=="":
			return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(data["language"],data["filter"])))

		return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(data["language"],data["filter"]))+"?search="+data["search"])

	# Si en cambio, no se ha enviado nada por POST,
	# cogemos todo de la URL
	LANGUAGES = dict(lang for lang in settings.LANGUAGES)
	if language not in LANGUAGES.keys():
		raise Http404(u"El idioma {0} no existe".format(language))
	
	if language == settings.LANGUAGE_CODE:
		raise Http404(u"El idioma {0} es el idioma por defecto".format(language))

	# Filtro para las traducciones
	trans_filter = {"lang":language}
	if filter == "all" or filter is None:
		pass
	elif filter == "fuzzy":
		trans_filter["is_fuzzy"] = True
	elif filter == "completed":
		trans_filter["is_fuzzy"] = False
	
	search_query = ""
	if request.GET and "search" in request.GET and request.GET.get("search")!="":
		search_query = request.GET.get("search")
		trans_filter["source_text__icontains"] = search_query

	translations = FieldTranslation.objects.filter(**trans_filter)
	
	# Actualiza las traducciones por si ha habido algún cambio en las
	# estructuras de los modelos referenciados y se tienen campos
	# de modelos que ya no los tienen
	active_translations = []
	for translation in translations:
		source_model = translation.get_source_model()
		if not translation.field in source_model._meta.translatable_fields:
			translation.delete()
		else:
			active_translations.append(translation)
	
	replacements = {"translations":active_translations, "filter":filter, "lang":language, "language":LANGUAGES[language], "search_query":search_query}
	return render_to_response('modeltranslation/admin/list.html',replacements, RequestContext(request))


########################################################################
########################################################################
# Editar una traducción al concreto
def edit(request, translation):
	"""
	Edita una traducción.
	"""
	translation = get_object_or_404(FieldTranslation, id=translation)
	if request.method == 'POST':
		if "cancel" in request.POST:
			return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(translation.lang,"all")))
		elif "save" in request.POST:
			form = FieldTranslationForm(request.POST, instance=translation)
			valid_form = form.is_valid()
			if valid_form:
				translation = form.save(commit=False)
				translation.context = u"Admin. Traducciones"
				translation.save()
				return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(translation.lang,"all")))
		else:
			form = FieldTranslationForm(instance=translation)
	else:
		form = FieldTranslationForm(instance=translation)

	LANGUAGES = dict(lang for lang in settings.LANGUAGES)
	language = LANGUAGES[translation.lang]
	
	return render_to_response('modeltranslation/admin/edit_translation.html',{"translation":translation, "form":form, "lang":translation.lang, "language":language}, RequestContext(request))


########################################################################
########################################################################
## Importar traducción en formato .po
@transaction.atomic
def import_translations(request, language):
	"""
	Importa las traducciones a partir de un archivo PO. Ten en cuenta
	que el archivo PO ha de ser generado desde esta aplicación, de forma
	que los comentarios sirvan como id de traducción (lo metemos nosotros
	en la exportación).
	"""
	def _import_po_file(uploadedfile, lang):
		lines = []
		for line in uploadedfile:
			lines.append(line)
		num_lines = len(lines)
		
		prog_ctxt = re.compile(r"msgctxt\s+\"(?P<id>\d+)--(?P<model>\w+)--(?P<object_id>\d+)--(?P<field>\w+)\"")
		prog_msgid = re.compile(r"msgid\s+\"(?P<source_text>.+)\"$")
		prog_msgstr = re.compile(r"msgstr\s+(?P<trans>.+)")
		
		i = 0
		while i < num_lines:
			line = lines[i]
			result = prog_ctxt.match(line)
			if result:
				id = result.group("id")
				is_fuzzy = (lines[i-1] == "#, fuzzy\n")

				source_text = lines[i+1]
				translation_line = lines[i+2]

				# Traducción
				g = prog_msgstr.match(translation_line)
				if g is None:
					i += 1
					continue
				translation = g.group("trans").replace("msgstr","")[1:-1].replace("\\\"","\"").replace('\\\'','\'')
					
				# Obtención de la traducción a partir del id
				try:
					field_trans = FieldTranslation.objects.get(id=id)
				except FieldTranslation.DoesNotExist:
					source_text = source_text.replace("msgid","")[1:-1].replace("\\\"","\"").replace('\\\'','\'')
					source_md5 = hashlib.md5(source_text.encode("utf-8")).hexdigest()
					field_trans = FieldTranslation(model=result.group("model"), object_id=result.group("object_id"), field=result.group("field"), lang=lang, source_text=source_text, source_md5=source_md5)
					
				# Establecemos la traducción y si es fuzzy
				field_trans.translation = translation
				field_trans.is_fuzzy = is_fuzzy
				field_trans.save()
				#print translation
				#print is_fuzzy
				i += 4
			i += 1
	
	# Elimina traducciones que no estén asociadas a ningún objeto
	FieldTranslation.delete_orphan_translations()
	
	# Acceso obligatoriamente por POST
	if request.method != "POST":
		return HttpResponseRedirect(reverse("modeltranslation:admin_url"))
	
	form = ImportTranslationsForm(request.POST, request.FILES)
	if form.is_valid():
		_import_po_file(request.FILES['file'], language)
		#cache = TransCache.factory()
		#cache.clear()
		return HttpResponseRedirect(reverse("modeltranslation:view_all_url",args=(language,"all")))

	return HttpResponseRedirect(reverse("modeltranslation:admin_url"))


########################################################################
########################################################################
## Exportar traducción en formato .po
def export_translations(request, language):
	"""
	Vista de exportación de las traducciones
	"""
	FieldTranslation.delete_orphan_translations()
	translations = FieldTranslation.objects.filter(lang=language)
	for trans in translations:
		trans.source_text = trans.source_text.replace("'","\'").replace("\"","\\\"")
		trans.translation = trans.translation.replace("'","\'").replace("\"","\\\"")
	replacements = {"translations":translations, "lang":language}
	if len(settings.ADMINS)>0:
		replacements["last_translator"] = settings.ADMINS[0][0]
		replacements["last_translator_email"] = settings.ADMINS[0][1]
	if settings.WEBSITE_NAME:
		replacements["website_name"] = settings.WEBSITE_NAME
	response = render(request=request, template_name='modeltranslation/admin/export_translations.po', dictionary=replacements, context_instance=RequestContext(request), content_type="text/x-gettext-translation")
	response['Content-Disposition'] = 'attachment; filename="{0}.po"'.format(language)
	return response


########################################################################
########################################################################
## Actualizar las traducciones
def update_translations(request):
	"""
	Actualiza las traducciones eliminando las huérfanas y
	generando traducciones vacías para todos los objetos que existan en
	base de datos.
	"""
	FieldTranslation.delete_orphan_translations()
	num_translations = FieldTranslation.update_translations()
	return render_to_response('modeltranslation/admin/update_translations_ok.html',{"num_translations":num_translations}, RequestContext(request))
	



# -*- coding: utf-8 -*-

import copy

from django import forms
from django.conf import settings
#from django.utils.translation import ugettext as _

from modeltranslation.models import trans_attr, trans_is_fuzzy_attr


########################################################################
## Useful ModelForm to translatable models
class TranslatableModelForm(forms.ModelForm):

	####################################################################
	## Automatically add new fields for each language you have in your system
	def _add_translation_form_fields(self):
		# Model of the ModelForm
		cls = self._meta.model
		# If Model has no translatable fields
		if not hasattr(cls,"_meta") or not hasattr(cls._meta,"translatable_fields") or len(cls._meta.translatable_fields)==0:
			return False

		# If it is an edition, load the current translations of the object
		if self.instance:
			self.instance.load_translations()

		# For each translatable field a new field for each language is created
		translatable_fields = cls._meta.translatable_fields

		_ = lambda x : x

		for translatable_field in translatable_fields:
			# Add additional language fields only if that field is present
			# in the ModelForm
			if (hasattr(self._meta, "fields") and self._meta.fields and translatable_field in self._meta.fields) or\
				(hasattr(self._meta, "exclude") and self._meta.exclude and not translatable_field in self._meta.exclude):

				self.fields[translatable_field].widget.is_translatable = True
				self.fields[translatable_field].widget.translation_group = translatable_field
				self.fields[translatable_field].widget.lang = settings.LANGUAGE_CODE

				if not settings.IS_MONOLINGUAL:
					for language_pair in settings.LANGUAGES:
						lang = language_pair[0]
						language_name = language_pair[1]

						if lang != settings.LANGUAGE_CODE:
							# Adds a translatable field
							# It is empty by default, its language must not be current language and
							# it should not be required
							field_lang = trans_attr(translatable_field, lang)
							self.fields[field_lang] = copy.deepcopy(self.fields[translatable_field])
							self.fields[field_lang].label += u" ({0})".format(language_name)
							self.fields[field_lang].initial = ""
							self.fields[field_lang].widget.lang = lang
							self.fields[field_lang].required = False

							# Original field label
							field_label = self.fields[translatable_field].label

							# If we are editing a Model instance, sets its correct initial values
							if self.instance and hasattr(self.instance, field_lang):
								self.fields[field_lang].initial = getattr(self.instance, field_lang)

							# is_fuzzy fields
							isfuzzy_lang = trans_is_fuzzy_attr(translatable_field,lang)
							self.fields[isfuzzy_lang] = forms.ChoiceField(
								choices=(
									(u"0",_(u"No necesita revisión")),
									(u"1", _(u"Necesita revisión"))),
								label=_(u"'{0}' necesita revisión para idioma {1}").format(field_label, language_name), initial="1")
							self.fields[isfuzzy_lang].widget.attrs["class"] = "is_fuzzy"
							if self.instance and hasattr(self.instance, isfuzzy_lang):
								if getattr(self.instance, isfuzzy_lang):
									self.fields[isfuzzy_lang].initial = "1"
								else:
									self.fields[isfuzzy_lang].initial = "0"
		return True


	####################################################################################################################
	## By default, adds the translation fields in the form.
	## Please, take note that if you modify the fields, you have to re-call _add_translation_form_fields to take effect
	## in the new fields
	def __init__(self, *args, **kwargs):
		super(TranslatableModelForm, self).__init__(*args, **kwargs)
		# Translations of the Model fields
		self._add_translation_form_fields()


	####################################################################################################################
	## Assigns translation attributes as dynamic attributes for the new instance or existing instance being edited.
	def save(self, commit=True):
		obj = super(TranslatableModelForm, self).save(commit)
		# The instance has all the translated fields introduced as dynamic attributes
		if hasattr(obj._meta, "translatable_fields") and hasattr(obj, "set_translation_fields"):
			# Set translation fields to object
			obj.set_translation_fields(self.cleaned_data)
			if commit:
				obj.save()
		return obj



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-01-19 12:21
from __future__ import unicode_literals
import importlib

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion

########################################################################################################################
## For each fieldtranslation we have to assign its module.
## Please, take note that we can do it now because we don't have models with the same name in different modules
## so we are going to fix this error before that situation happens.
def assign_module(apps, schema_editor):
    # Get the current model
    FieldTranslation = apps.get_model('modeltranslation', 'FieldTranslation')

    ## For each fieldtranslation we have to assign its module.
    field_translations = FieldTranslation.objects.all()
    for field_translation in field_translations:
        # For each module with translatable models, we load it
        # and test if the model is inside this module
        # It could be made more efficiently but there are no many developers currently using this application so
        # we don't have to be extremely careful about performance.
        for module_path in settings.TRANSLATABLE_MODEL_MODULES:
            module = importlib.import_module(module_path)
            if hasattr(module, field_translation.model):
                field_translation.module = module_path
                field_translation.save()


class Migration(migrations.Migration):

    dependencies = [
        ('modeltranslation', '0005_auto_20160107_1058'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='fieldtranslation',
            options={'verbose_name': 'model object field translation', 'verbose_name_plural': 'model object field translations'},
        ),
        migrations.AddField(
            model_name='fieldtranslation',
            name='module',
            field=models.CharField(default='', help_text='Module name that contains the model whose field is translated', max_length=128, verbose_name='Module name'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='context',
            field=models.TextField(default=None, help_text='Help context that will be helpful for translators.', null=True, verbose_name='Context'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='creation_datetime',
            field=models.DateTimeField(verbose_name='Creation date and time of this translation'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='creator_user',
            field=models.ForeignKey(default=None, help_text='User that created last translation version', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='model_translation', to=settings.AUTH_USER_MODEL, verbose_name='User translator'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='field',
            field=models.CharField(help_text='Name of the field that is translated', max_length=128, verbose_name='Object field'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='is_fuzzy',
            field=models.BooleanField(default=False, help_text='This translation needs some reviewing.', verbose_name='\xbfNeeds reviewing?'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='lang',
            field=models.CharField(choices=[('es', 'Espa\xf1ol'), ('en', 'English'), ('de', 'Deutsch')], help_text='Language ISO code of this translation', max_length=16, verbose_name='Language Id'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='last_update_datetime',
            field=models.DateTimeField(verbose_name='Last update date and time of this translation'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='model',
            field=models.CharField(help_text='Model name whose field is translated', max_length=128, verbose_name='Model name'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='object_id',
            field=models.PositiveIntegerField(default=1, help_text='Object id whose field is translated', verbose_name='Object id'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='source_md5',
            field=models.CharField(help_text='MD5 checksum of source text', max_length=128, verbose_name='MD5 source text'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='source_text',
            field=models.TextField(help_text='Source text in default language', verbose_name='Source text'),
        ),
        migrations.AlterField(
            model_name='fieldtranslation',
            name='translation',
            field=models.TextField(default=None, help_text='Translation showed to users in website when showing it in choosed language.', null=True, verbose_name='Translation'),
        ),
        migrations.RunPython(assign_module)
    ]



# -*- coding: utf-8 -*-

from django.conf.urls import url

# ModelTranslationURLs
from modeltranslation.admin import views


class ModelTranslationUrls(object):
	def __init__(self, name='modeltranslation', app_name='modeltranslation'):
		self.app_name = app_name # App Namespace
		self.name = name # Instance Namespace

	urlpatterns = [
		url(r'^$', views.admin, name="admin_url"),
		url(r'^(?P<language>[-\w]+)/(?P<filter>(all|completed|fuzzy)*)$', views.view_all, name="view_all_url"), # Translation management
		url(r'^edit_translation/(?P<translation>.*)$', views.edit, name="edit_url"), # Edition
		url(r'^import/(?P<language>[-\w]+)$', views.import_translations, name="import_translations_url"), # Translation import
		url(r'^export/(?P<language>[-\w]+)$', views.export_translations, name="export_translations_url"), # Translation export
		url(r'^update_translations$', views.update_translations, name="update_translations_url"), # Update translations
	]

	@property
	def urls(self):
		return (self.urlpatterns, self.app_name, self.name)
